﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Input;
using B3.PluginAPIKit;
using System.Xml;

namespace InsertElementPlugin
{
	public class InsertElementPlugin : IPlugin
	{
		private IHost mHost;
		private bool mActivated;
		private INavContext mNav;
		private IEditorContext mEditorContext;

		public bool IsActivated
		{
			get { return mActivated; }
		}

		public string GetVersion()
		{
			return "0.1";
		}

		public string GetAuthor()
		{
			return "Flare for Programmers";
		}

		public string GetDescription()
		{
			return "This is a change element tag plug-in.";
		}

		public string GetName()
		{
			return "Change Element Tag";
		}

		public void Initialize(IHost host)
		{
			mHost = host;
		}

		public void Execute()
		{
			mActivated = true;
			mNav = mHost.GetNavContext();
			mEditorContext = mHost.GetEditorContext();
			CreateToolBarAndRibbon();
		}

		public void Stop()
		{
			MessageBox.Show(GetName() + " deactivated.");
			mHost.Dispose();
			mActivated = false;
		}

		private void CreateToolBarAndRibbon()
		{
			//ToolBar
			ICustomToolBar toolbar = mNav.CreateCustomToolBar("Element Tags");
			toolbar.AddButton("Change", new ButtonCommand(mEditorContext));

			//Ribbon
			IRibbon ribbon = mNav.GetRibbon();
			IRibbonTab tab = ribbon.AddNewRibbonTab("Element Tags");
			IRibbonGroup group = tab.AddNewRibbonGroup("Element Tags");
			group.AddRibbonButton("Change", 
				new ButtonCommand(mEditorContext), 
				null, null, RibbonIconSize.Collapsed, null, null);
		}

		public static void buttonActions(IEditorContext pEditorContext)
		{
			IDocument activeDocument = pEditorContext.GetActiveDocument();
			ISelection selection;

			//We only want to change in the active document. If there isn't one, do nothing.
			if (activeDocument != null)
			{
				//Get the nodes in the selection and find the last element.
				selection = activeDocument.Selection;
				IList<XmlNode> nodes = selection.GetXmlNodeList();
				XmlNode node;
				if (nodes.Last().NodeType.Equals(System.Xml.XmlNodeType.Element))
				{
					node = nodes.Last();
				}
				else
				{
					if (nodes[0].ParentNode.Equals(null))
					{
						node = nodes.Last();
					}
					else
					{
						node = nodes.Last().ParentNode;
					}
				}

				//Get the new tag name
				string input = Microsoft.VisualBasic.Interaction.InputBox("Enter new tag",
					"Tag", "Default", -1, -1);

				//Create the new element and use ReplaceDocumentNode
				XmlElement el = (XmlElement)node;
				XmlAttributeCollection attrs = el.Attributes;
				string inner = el.InnerXml;
				XmlElement newEl = activeDocument.GetXmlDocument().CreateElement(input);
				newEl.InnerXml = inner;
				activeDocument.SetCursorPosition(0);
				activeDocument.Selection.SetSelectionLength(0);
				activeDocument.ReplaceDocumentNode(node, newEl, true);
				//activeDocument.InsertDocumentNode(node,
				//	newEl, activeDocument.GetCursorPosition(), true);
				activeDocument.UpdateView();
			}
		}

		private class ButtonCommand : ICommand
		{
			private IEditorContext passEditorContext;

			public ButtonCommand(IEditorContext pEditorContext)
			{
				passEditorContext = pEditorContext;
			}

			public bool CanExecute(object parameter)
			{
				return true;
			}

			public event EventHandler CanExecuteChanged;

			public void Execute(object parameter)
			{
				buttonActions(passEditorContext);
			}
		}
	}
}
