﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Input;
using B3.PluginAPIKit;
using System.Xml;

namespace CodePastePlugin
{
	public class CodePastePlugin : IPlugin
	{
		private IHost mHost;
		private bool mActivated;
		private INavContext mNav;
		private IEditorContext mEditorContext;

		public bool IsActivated
		{
			get { return mActivated; }
		}

		public string GetVersion()
		{
			return "0.1";
		}

		public string GetAuthor()
		{
			return "Flare for Programmers";
		}

		public string GetDescription()
		{
			return "This is a code paste plug-in.";
		}

		public string GetName()
		{
			return "Code Paste";
		}

		public void Initialize(IHost host)
		{
			mHost = host;
		}

		public void Execute()
		{
			mActivated = true;
			mNav = mHost.GetNavContext();
			mEditorContext = mHost.GetEditorContext();
			CreateToolBarAndRibbon();
		}

		public void Stop()
		{
			MessageBox.Show(GetName() + " deactivated.");
			mHost.Dispose();
			mActivated = false;
		}

		private void CreateToolBarAndRibbon()
		{
			//ToolBar
			ICustomToolBar toolbar = mNav.CreateCustomToolBar("Code Paste");
			toolbar.AddButton("Paste", new ButtonCommand(mEditorContext));

			//Ribbon
			IRibbon ribbon = mNav.GetRibbon();
			IRibbonTab tab = ribbon.AddNewRibbonTab("Code Paste");
			IRibbonGroup group = tab.AddNewRibbonGroup("Code Paste");
			group.AddRibbonButton("Paste", new ButtonCommand(mEditorContext),
				null, null, RibbonIconSize.Collapsed, null, null);
		}

		public static void buttonActions(IEditorContext pEditorContext)
		{
			IDocument activeDocument = pEditorContext.GetActiveDocument();
			ISelection selection;

			//We only want to insert into the active document.
			//If there isn't one, do nothing.
			if (activeDocument != null)
			{
				//Get the nodes in the selection and find the last element.
				selection = activeDocument.Selection;
				IList<XmlNode> nodes = selection.GetXmlNodeList();
				XmlNode parentNode;
				if (nodes.Last().NodeType.Equals(System.Xml.XmlNodeType.Element))
				{
					parentNode = nodes.Last();
				}
				else
				{
					if (nodes[0].ParentNode.Equals(null))
					{
						parentNode = nodes.Last();
					}
					else
					{
						parentNode = nodes.Last().ParentNode;
					}
				}
				//Create a code element that contains copied text.
				XmlElement el =
					activeDocument.GetXmlDocument().CreateElement("code");
				el.InnerText =
					Clipboard.GetText(System.Windows.Forms.TextDataFormat.UnicodeText);
				//Insert the code element
				//at the end of the last element and update the view.
				activeDocument.InsertDocumentNode(parentNode, (XmlNode)el,
					activeDocument.GetCursorPosition(), false);
				activeDocument.UpdateView();
			}
		}


		private class ButtonCommand : ICommand
		{
			private IEditorContext passEditorContext;

			public ButtonCommand(IEditorContext pEditorContext)
			{
				passEditorContext = pEditorContext;
			}

			public bool CanExecute(object parameter)
			{
				return true;
			}

			public event EventHandler CanExecuteChanged;

			public void Execute(object parameter)
			{
				buttonActions(passEditorContext);
			}
		}
	}
}
