package flarecontentgenerator;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class FlareContentGenerator {

	public static Document returnTopic() throws ParserConfigurationException,
			SAXException, IOException {
		DocumentBuilderFactory documentFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder documentBuilder = documentFactory.newDocumentBuilder();

		String topicString = "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
				+ "<html xmlns:MadCap=\"http://www.madcapsoftware.com/Schemas/MadCap.xsd\">"
				+ "<head />" + "<body />" + "</html>";

		Document topic = documentBuilder.parse(new InputSource(
				new StringReader(topicString)));
		return topic;
	}

	public static Document returnTopic(String title, String text)
			throws ParserConfigurationException, SAXException, IOException {
		Document topic = returnTopic();
		Element titleElement = topic.createElement("title");
		titleElement.setTextContent(title);
		topic.getElementsByTagName("head").item(0).appendChild(titleElement);
		Element headingElement = topic.createElement("h1");
		headingElement.setTextContent(title);
		topic.getElementsByTagName("body").item(0).appendChild(headingElement);
		Element paragraph = topic.createElement("p");
		paragraph.setTextContent(text);
		topic.getElementsByTagName("body").item(0).appendChild(paragraph);
		return topic;
	}

	public static Document returnEmptyToc()
			throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory documentFactory = DocumentBuilderFactory
				.newInstance();
		DocumentBuilder documentBuilder = documentFactory.newDocumentBuilder();

		String tocString = "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
				+ "<CatapultToc Version=\"1\" />";

		Document toc = documentBuilder.parse(new InputSource(new StringReader(
				tocString)));
		return toc;
	}

	public static Document returnFlatToc(List<Element> tocEntries)
			throws ParserConfigurationException, SAXException, IOException {
		Document toc = returnEmptyToc();
		for (Element t : tocEntries) {
			toc.getElementsByTagName("CatapultToc").item(0).appendChild(t);
		}
		return toc;
	}

	public static Element returnTocEntry(Document toc, String topicLabel,
			String topicLink) {
		Element tocEntry = toc.createElement("TocEntry");
		tocEntry.setAttribute("Title", topicLabel);
		tocEntry.setAttribute("Link", topicLink);
		return tocEntry;
	}

	public static Element returnTocEntryAppendChildEntry(Document toc,
			Element tocEntry, String topicLabel, String topicLink) {
		Element childTocEntry = returnTocEntry(toc, topicLabel, topicLink);
		tocEntry.appendChild(childTocEntry);
		return tocEntry;
	}

	public static Element returnTocEntryAppendChildEntry(Document toc,
			Element tocEntry, Element childTocEntry) {
		tocEntry.appendChild(childTocEntry);
		return tocEntry;
	}

	public static Document returnTocAppendTocEntry(Document toc, String topicLabel,
			String topicLink) {
		Element tocEntry = returnTocEntry(toc, topicLabel, topicLink);
		toc.getElementsByTagName("CatapultToc").item(0).appendChild(tocEntry);
		return toc;
	}

	public static Document returnTocAppendTocEntry(Document toc,
			Element tocEntry) {
		toc.getElementsByTagName("CatapultToc").item(0).appendChild(tocEntry);
		return toc;
	}

	public static void main(String[] args) throws ParserConfigurationException,
			SAXException, IOException, TransformerException {
		Document firstTopic = returnTopic();
		String firstTopicTitle = "First Topic";
		String firstTopicFilename = "FirstTopic.htm";
		String firstTopicLink = "/Content/" + firstTopicFilename;
		String firstTopicText = "This is some text for the first topic.";
		firstTopic = returnTopic(firstTopicTitle, firstTopicText);

		String secondTopicTitle = "Second Topic";
		String secondTopicFilename = "SecondTopic.htm";
		String secondTopicLink = "/Content/" + secondTopicFilename;
		String secondTopicText = "This is some text for the second topic.";
		Document secondTopic = returnTopic(secondTopicTitle, secondTopicText);

		Document toc = returnEmptyToc();
		Element firstTocEntry = returnTocEntry(toc, firstTopicTitle,
				firstTopicLink);
		Element secondTocEntry = returnTocEntry(toc, secondTopicTitle,
				secondTopicLink);
		Element thirdTocEntry = returnTocEntryAppendChildEntry(toc,
				firstTocEntry, secondTocEntry);
		toc = returnTocAppendTocEntry(toc, thirdTocEntry);

		String basePathContent = "C:\\Examples\\ExampleProject\\Content\\";
		String basePathTOCs = "C:\\Examples\\ExampleProject\\Project\\TOCs\\";
		TransformerFactory transformerFactory = TransformerFactory
				.newInstance();
		Transformer transformer = transformerFactory.newTransformer();
		DOMSource source = new DOMSource(firstTopic);
		File file = new File(basePathContent + firstTopicFilename);
		StreamResult streamResult = new StreamResult(file);
		transformer.transform(source, streamResult);
		
		source = new DOMSource(secondTopic);
		file = new File(basePathContent + secondTopicFilename);
		streamResult = new StreamResult(file);
		transformer.transform(source, streamResult);
		
		Document emptyToc = returnEmptyToc();
		source = new DOMSource(emptyToc);
		streamResult = new StreamResult(new File(basePathTOCs + "empty.fltoc"));
		transformer.transform(source, streamResult);
		
		source = new DOMSource(toc);
		streamResult = new StreamResult(new File(basePathTOCs + "other.fltoc"));
		transformer.transform(source, streamResult);
	}
}
